using System.Text.Json;
using HermesSocketLibrary.db;
using HermesSocketLibrary.Requests.Messages;
using HermesSocketServer.Store.Internal;

namespace HermesSocketServer.Store
{
    public class ActionStore : ComplexAutoSavedStore<string, RedeemableAction>
    {
        private readonly string _userId;
        private readonly Database _database;
        private readonly Serilog.ILogger _logger;


        public ActionStore(string userId, DatabaseTable table, Database database, Serilog.ILogger logger)
            : base(table, database, logger)
        {
            _userId = userId;
            _database = database;
            _logger = logger;
        }

        public override async Task Load()
        {
            var data = new Dictionary<string, object>() { { "user", _userId } };
            string sql = $"SELECT name, type, has_message, data FROM \"Action\" WHERE \"userId\" = @user";
            await _database.Execute(sql, data, (reader) =>
            {
                var name = reader.GetString(0);
                _store.Add(name, new RedeemableAction()
                {
                    UserId = _userId,
                    Name = name,
                    Type = reader.GetString(1),
                    HasMessage = reader.GetBoolean(2),
                    Data = JsonSerializer.Deserialize<IDictionary<string, string>>(reader.GetString(3))!
                });
            });
            _logger.Information($"Loaded {_store.Count} redeemable actions from database.");
        }

        protected override void OnInitialAdd(string key, RedeemableAction value)
        {
            ArgumentException.ThrowIfNullOrWhiteSpace(key, nameof(key));
            ArgumentNullException.ThrowIfNull(value, nameof(value));
            ArgumentException.ThrowIfNullOrWhiteSpace(value.UserId, nameof(value.UserId));
            ArgumentException.ThrowIfNullOrWhiteSpace(value.Name, nameof(value.Name));
            ArgumentException.ThrowIfNullOrWhiteSpace(value.Type, nameof(value.Type));
            ArgumentNullException.ThrowIfNull(value.Data, nameof(value.Data));

            if (value.Name.Length > 36)
                throw new ArgumentException("Action name cannot be longer than 36 characters.");
        }

        protected override void OnInitialModify(string key, RedeemableAction oldValue, RedeemableAction newValue)
        {
            ArgumentNullException.ThrowIfNull(newValue, nameof(newValue));
            ArgumentException.ThrowIfNullOrWhiteSpace(newValue.UserId, nameof(newValue.UserId));
            ArgumentException.ThrowIfNullOrWhiteSpace(newValue.Name, nameof(newValue.Name));
            ArgumentException.ThrowIfNullOrWhiteSpace(newValue.Type, nameof(newValue.Type));
            ArgumentNullException.ThrowIfNull(newValue.Data, nameof(newValue.Data));
            ArgumentOutOfRangeException.ThrowIfNotEqual(oldValue.UserId, newValue.UserId, nameof(oldValue.UserId));
            ArgumentOutOfRangeException.ThrowIfNotEqual(oldValue.Name, newValue.Name, nameof(oldValue.Name));
        }
    }
}