using HermesSocketLibrary.db;
using HermesSocketLibrary.Requests.Messages;
using HermesSocketServer.Store.Internal;

namespace HermesSocketServer.Store
{
    public class TTSFilterStore : AutoSavedStore<string, TTSWordFilter>
    {
        private readonly string _userId;
        private readonly Database _database;
        private readonly Serilog.ILogger _logger;


        public TTSFilterStore(string userId, DatabaseTable table, Database database, Serilog.ILogger logger)
            : base(table, database, logger)
        {
            _userId = userId;
            _database = database;
            _logger = logger;
        }

        public override async Task Load()
        {
            var data = new Dictionary<string, object>() { { "user", _userId } };
            string sql = $"SELECT id, search, replace, flag FROM \"TtsWordFilter\" WHERE \"userId\" = @user";
            await _database.Execute(sql, data, (reader) =>
            {
                var id = reader.GetString(0);
                _store.Add(id.ToString(), new TTSWordFilter()
                {
                    Id = id,
                    UserId = _userId,
                    Search = reader.GetString(1),
                    Replace = reader.GetString(2),
                    Flag = reader.GetInt32(3),
                });
            });
            _logger.Information($"Loaded {_store.Count} TTS filters from database.");
        }

        protected override void OnInitialAdd(string key, TTSWordFilter value)
        {
            ArgumentException.ThrowIfNullOrWhiteSpace(key, nameof(key));
            ArgumentNullException.ThrowIfNull(value, nameof(value));
            ArgumentException.ThrowIfNullOrWhiteSpace(value.Id, nameof(value.Id));
            ArgumentException.ThrowIfNullOrWhiteSpace(value.UserId, nameof(value.UserId));
            ArgumentException.ThrowIfNullOrWhiteSpace(value.Search, nameof(value.Search));
            ArgumentNullException.ThrowIfNull(value.Replace, nameof(value.Replace));
            ArgumentOutOfRangeException.ThrowIfNegative(value.Flag, nameof(value.Flag));
            ArgumentOutOfRangeException.ThrowIfGreaterThan(value.Flag, 2047, nameof(value.Flag));
        }

        protected override void OnInitialModify(string key, TTSWordFilter oldValue, TTSWordFilter newValue)
        {
            ArgumentNullException.ThrowIfNull(newValue, nameof(newValue));
            ArgumentException.ThrowIfNullOrWhiteSpace(newValue.Id, nameof(newValue.Id));
            ArgumentException.ThrowIfNullOrWhiteSpace(newValue.UserId, nameof(newValue.UserId));
            ArgumentException.ThrowIfNullOrWhiteSpace(newValue.Search, nameof(newValue.Search));
            ArgumentNullException.ThrowIfNull(newValue.Replace, nameof(newValue.Replace));
            ArgumentOutOfRangeException.ThrowIfNegative(newValue.Flag, nameof(newValue.Flag));
            ArgumentOutOfRangeException.ThrowIfGreaterThan(newValue.Flag, 2047, nameof(newValue.Flag));
            ArgumentOutOfRangeException.ThrowIfNotEqual(oldValue.Id, newValue.Id, nameof(oldValue.Id));
            ArgumentOutOfRangeException.ThrowIfNotEqual(oldValue.UserId, newValue.UserId, nameof(oldValue.UserId));
        }

        protected override void OnPostRemove(string key, TTSWordFilter value)
        {
        }
    }
}